/*
    UPN-Calc - An arbitrary precision RPN calculator for J2ME/MIDP1
    Copyright (C) 2005 Kai G. Schwebke, upn.schwebke.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/*
 * Floating Point Constants
 */

class BCDConst
{
   private static BCDFloat one;
   private static BCDFloat two;
   private static BCDFloat half;
   private static BCDFloat ten;
   private static BCDFloat e;
   private static BCDFloat eInv;
   private static BCDFloat ln10;
   private static BCDFloat pi;
   private static BCDFloat pi2;
   private static BCDFloat pi_2;


   // call after reset of precision
   // to enforce recalculation of
   // affected constants
   public static void reset()
   {
      e = eInv = ln10 = pi = pi2 = pi_2 = null;
   }

   public static BCDFloat getOne()
   {
      if (one == null)
      {
	 one = new BCDFloat("1");
      }
      return one;
   }

   public static BCDFloat getHalf()
   {
      if (half == null)
      {
	 half = new BCDFloat("0.5");
      }
      return half;
   }

   public static BCDFloat getTwo()
   {
      if (two == null)
      {
	 two = new BCDFloat("2");
      }
      return two;
   }

   public static BCDFloat getTen()
   {
      if (ten == null)
      {
	 ten = new BCDFloat("10");
      }
      return ten;
   }

   public static BCDFloat getE()
   {
      if (e == null)
      {
	 e = getOne().expI();
      }
      return e;
   }

   public static BCDFloat getEInv()
   {
      if (eInv == null)
      {
	 eInv = getOne().div(getE());
      }
      return eInv;
   }

   public static BCDFloat getLn10()
   {
      if (ln10 == null)
      {
	 ln10 = getTen().ln();
      }
      return ln10;
   }

   public static BCDFloat getPi()
   {
      if (pi == null)
      {
	 // FIXME: this is not truly arbitrary...
	 pi = new BCDFloat("3.1415926535897932384626433"+
	                   "832795028841971693993751058"+
			   "209749445923078164062862089"+
			   "986280348253421170679821480"+
			   "865132823066470938446095505"+
			   "822317253594081284811174502"+
			   "841027019385211055596446229"+
			   "489549303820");
	 pi.round();
      }
      return pi;
   }

   public static BCDFloat get2Pi()
   {
      if (pi2 == null)
      {
	 pi2 = getPi().mul(getTwo());
      }
      return pi2;
   }

   public static BCDFloat getPiHalf()
   {
      if (pi_2 == null)
      {
	 pi_2 = getPi().mul(getHalf());
      }
      return pi_2;
   }

}
